/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Netscape Security Services for Java.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are 
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All
 * Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */

package org.mozilla.jss.pkcs11;

import java.util.*;
import org.mozilla.jss.util.*;
import org.mozilla.jss.crypto.*;

/**
 * A random number generator for PKCS #11.
 *
 * @see org.mozilla.jss.CryptoManager
 */
public final
class PK11SecureRandom implements org.mozilla.jss.crypto.JSSSecureRandom
{
    ////////////////////////////////////////////////////
    // construction and finalization
    ////////////////////////////////////////////////////

    public
    PK11SecureRandom() {}

    ////////////////////////////////////////////////////
    //  public routines
    ////////////////////////////////////////////////////

    public synchronized native void
    setSeed( byte[] seed );

    public void
    setSeed( long seed )
    {
        byte[] data = new byte[8];

        // convert long into 8-byte byte array
        for( int i = 0; i < 8; i++ ) {
             data[i] = ( byte ) ( seed >> ( 8 * i ) );
        }

        setSeed( data );
    }

    public synchronized native void
    nextBytes( byte bytes[] );
}

